/* utilities.cpp
   implementation for utility function
   @author	Hyun Kim
   @date	11/20/99
*/

#include "utilities.hpp"
#include "random.hpp"
#include "Parameter.hpp"
#include "Chromosome.hpp"
#include "Population.hpp"
#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>

/* sort the population by fitness( bubble sort )
   @param pop poulation
   @param arr indices of all chromosomes in the population
   @param size size of the population
*/
void sort( Population& pop, int* arr, int size )
{
	int end = size - 1;
	int i;
	for ( end = size - 1; end >= 1; end-- )
	{
		for ( i = 0; i <= end - 1; i++ )
		{
			if ( pop[ arr[ i ] ].getFitness() > pop[ arr[ i + 1 ] ].getFitness() )
			{
				swap( arr[ i ], arr[ i + 1 ] );
			}
		}
	}
}

/* swap two integer values
   @param d1 one value
   @param d2 another value
*/
void swap( int& d1, int& d2 )
{
	int tmp;
	tmp = d1;
	d1 = d2;
	d2 = tmp;
}

/* swap two double values
   @param d1 one value
   @param d2 another value
*/
void swap( double& d1, double& d2 )
{
	double tmp;
	tmp = d1;
	d1 = d2;
	d2 = tmp;
}

/* compute the square of the value
   @param d value to be squared
   @return double
*/
double sqr( double& d )
{
	return d * d;
}

/* compute the expected return of the chromosome
   @param chrom chromosome we compute the expected return
   @return double
*/
double getExpReturn( const Chromosome& chrom )
{
	double portfolioExpReturn = 0;

	for ( int i = 0; i < chrom.getLength(); i++ )
	{
		portfolioExpReturn += Parameter::expReturn[ i ] * chrom.getGenes( i );
	}
	return portfolioExpReturn;
}

/* compute the standard deviation of expected returns of the chromosome
   @param chrom chromosome we compute the std. dev.
   @return double
*/
double getStdDev( const Chromosome& chrom )
{
	double variance = 0;

	for ( int i = 0; i < chrom.getLength(); i++ )
	{
		variance += sqr( chrom.getGenes( i ) ) * sqr( Parameter::stdDevReturn[ i ] );
		for ( int j = i + 1; j < chrom.getLength(); j++ )
		{
			double covariance = Parameter::correlation[ i ][ j ] * Parameter::stdDevReturn[ i ] * Parameter::stdDevReturn[ j ];
			variance += 2 * chrom.getGenes( i ) * chrom.getGenes( j ) * covariance;
		}
	}
	return sqrt( variance );
}

/* print the chromosome to the console
   @param out console output stream
   @param chrom chromosome that we want to print
   @return ostream
*/
ostream& operator<<( ostream& out, Chromosome& chrom )
{
	for ( int i = 0; i < Parameter::genes; i++ )
	{
		out << chrom[ i ] << " ";
	}
	return out;
}

/* print the population to the console
   @param out console output stream
   @param pop population that we want to print
   @return ostream
*/
ostream& operator<<( ostream& out, Population& pop )
{
	for ( int i = 0; i < Parameter::popsize; i++ )
	{
		out << pop[ i ] << endl;
	}
	return out;
}

/* print the chromosome to the file
   @param out file output stream
   @param chrom chromosome that we want to print
   @return ostream
*/
ofstream& operator<<( ofstream& out, Chromosome& chrom )
{
	for ( int i = 0; i < Parameter::genes; i++ )
	{
		out	<< setprecision(7) << chrom[ i ] << " ";
	}
	return out;
}

/* print the population to the file
   @param out file output stream
   @param pop population that we want to print
   @return ostream
*/
ofstream& operator<<( ofstream& out, Population& pop )
{
	for ( int i = 0; i < Parameter::popsize; i++ )
	{
		out << pop[ i ] << endl;
	}
	return out;
}
